package com.ruoyi.psi.service.impl;

import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.psi.domain.*;
import com.ruoyi.psi.domain.vo.AddBiddingVo;
import com.ruoyi.psi.domain.vo.SupplyVO;
import com.ruoyi.psi.mapper.*;
import com.ruoyi.psi.service.IPsiSupplyService;
import com.ruoyi.psi.service.IBiddingService;
import com.ruoyi.system.domain.SysTenant;
import com.ruoyi.system.mapper.SysTenantMapper;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 竞价单Service业务层处理
 * 
 * @author ruoyi
 * @date 2023-12-17
 */
@Service
public class BiddingServiceImpl implements IBiddingService 
{
    @Autowired
    private BiddingMapper BiddingMapper;

    @Autowired
    private PsiPurchaseItemMapper purchaseItemMapper;

    @Autowired
    private PsiGoodsMapper goodsMapper;

    @Autowired
    private PsiSupplierMapper supplierMapper;

    @Autowired
    private BiddingItemMapper biddingItemMapper;

    @Autowired
    private SysTenantMapper tenantMapper;

    @Autowired
    private PsiPurchaseMapper psiPurchaseMapper;

    private static final Logger log = LoggerFactory.getLogger(BiddingServiceImpl.class);

    /**
     * @description 根据采购单id生成竞价单
     *
     * @param purchaseId
     * @param tenantId
     * @return int
     * @author fm
     * @date 2023/12/17 10:12
     */
    @Override
    @Transactional
    public List<PsiSupplier> getBiddingSupplier(Long purchaseId, Long tenantId) {
        // 获得采购单详情
        List<PsiPurchaseItem> purchaseItems = purchaseItemMapper.selectPsiPurchaseItemByPurchaseId(purchaseId);
        if (CollectionUtils.isEmpty(purchaseItems)) {
            throw new ServiceException("采购单不能为空!");
        }

        // 商品数量
        int goodsNum = purchaseItems.size();
        // 创建一个二维数组，存放明细中每个商品对应的商品列表
        // 二维数组 行：采购单明细索引   列：商品列表
        PsiGoods[][] goods_arr = new PsiGoods[goodsNum][];
        PsiGoods goods = null;
        // 遍历采购单，并根据商品名称+规格+租户 找出满足条件的商品列表，可能有多个供应商都提供 可乐+260ml，所以是多个
        for (int i = 0; i < purchaseItems.size(); i++) {
            goods = new PsiGoods();
            goods.setGoodsName(purchaseItems.get(i).getGoodsName());
            goods.setSpec(purchaseItems.get(i).getSpec());
            goods.setTenantId(tenantId);
            // 当前租户 + 规格 + 商品名称 获得数据
            List<PsiGoods> psiGoodsList = goodsMapper.selectGoodsByPurchase(goods);
            if (CollectionUtils.isEmpty(psiGoodsList)) {
                throw new ServiceException(purchaseItems.get(i).getGoodsName() + "未找到此商品!");
            }
            goods_arr[i] = psiGoodsList.toArray(new PsiGoods[0]);
        }

        // 获取供应商交集，即可以提供采购单中全部商品的供应商ID
        // 首先创建一个HashSet来存储交集元素,将第一个数组goods_arr[0]的元素添加到集合中
        Set<Long> intersection = new HashSet<>();
        for (int i = 0; i < goods_arr[0].length; i++) {
            intersection.add(goods_arr[0][i].getSupplierId());
        }
        // 然后，遍历剩余的数组，并使用retainAll方法计算交集，这个方法会保留集合中与当前数组元素相同的元素。
        for (int i = 1; i < goods_arr.length; i++) {
            Set<Long> currentSet = new HashSet<>();
            for (int j = 0; j < goods_arr[i].length; j++) {
                currentSet.add(goods_arr[i][j].getSupplierId());
            }
            intersection.retainAll(currentSet);
        }
        log.debug("竞价模式，匹配条件的供应商ID：" + intersection);
        if (CollectionUtils.isEmpty(intersection)) {
            throw new ServiceException("未找到符合可以报价的供应商，没有1个供应商可以提供所有商品!");
        }
        List<PsiSupplier> supplierList = supplierMapper.selectPsiSupplierListByIds(intersection.toArray(new Long[0]));

        return supplierList;
    }

    /**
     * @description 根据采购单id生成竞价单
     *
     * @param vo
     * @return int
     * @author fm
     * @date 2023/12/17 10:12
     */
    @Override
    @Transactional
    public int addBidding(AddBiddingVo vo) {
        Long purchaseId = vo.getPurchaseId();
        Long tenantId = vo.getTenantId();
        Long[] suppliers = vo.getSupplierIds();
        // 获得采购单详情
        List<PsiPurchaseItem> purchaseItems = purchaseItemMapper.selectPsiPurchaseItemByPurchaseId(purchaseId);
        if(CollectionUtils.isEmpty(purchaseItems)){
            throw new ServiceException("采购单不能为空!");
        }

        // 生成竞价单
        Long userId = SecurityUtils.getLoginUser().getUser().getUserId();
        String nickName = SecurityUtils.getLoginUser().getUser().getNickName();
        SysTenant sysTenant = tenantMapper.selectSysTenantById(tenantId);
        String tenantName = sysTenant.getTenantName();
        // 为每个符合条件的供应商生产报价单
        for (int s=0; s<suppliers.length; s++) {
            Bidding bidding = new Bidding();
            bidding.setCreateUser(userId);
            bidding.setCreateTime(DateUtils.getNowDate());
            bidding.setSupplierId(suppliers[s]);
            bidding.setTenantId(tenantId);
            bidding.setPurchaseId(purchaseId);
            bidding.setStatus(0);
            bidding.setCreateUsername(nickName);
            bidding.setTenantName(tenantName);
            BiddingMapper.insertBidding(bidding);

            Long biddingId = bidding.getId();

            BiddingItem biddingItem = null;
            List<BiddingItem> biddingItems = new ArrayList<>();
            for (int i = 0; i < purchaseItems.size(); i++) {
                biddingItem = new BiddingItem();
                BeanUtils.copyProperties(purchaseItems.get(i),biddingItem);
                biddingItem.setBiddingId(biddingId);

                PsiGoods psiGoods = goodsMapper.selectOneGoodsByNameAndSpec(biddingItem.getGoodsName(), biddingItem.getSpec().toLowerCase(), suppliers[s], tenantId);
                biddingItem.setGoodsId(psiGoods.getId());
                biddingItem.setPrice(psiGoods.getPrice());
                biddingItems.add(biddingItem);
            }
            // 插入详细表
            BiddingMapper.savaBathList(biddingItems,biddingId);
            PsiPurchase purchase = new PsiPurchase();
            purchase.setId(purchaseId);
            purchase.setTenantId(tenantId);
            purchase.setStatus(3);
            psiPurchaseMapper.updatePsiPurchase(purchase);
        }
        return 1;
    }
}
